import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.beans.*;
import javax.swing.plaf.basic.BasicInternalFrameUI;
import java.util.*;


// this class displays all visuals
public class ClientFrame extends JFrame implements Observer {
  private static final int WIDTH = 800;
  private static final int HEIGHT = 700;
  
  private String user_name, room_title;
  
  // default server info
  private String server_ip = "71.231.78.161";
  private String port = "1500";
  
  private ClientNetwork cn;
  private DrawingCanvas canvas;
  
  private int game_mode = -1; // no mode
  
  private  JInternalFrame iframe_login= new JInternalFrame("", false, false, false, false);
  private  JInternalFrame iframe_wait;
  private  JInternalFrame iframe_play;
  private JDesktopPane desktop= new JDesktopPane();
  JTextArea chatDisplay = new JTextArea();
  
  JTextField[] playerList = new JTextField[6];
  
  public ClientFrame() {
    
    setTitle("OpenSpace Beta");
    setSize(WIDTH, HEIGHT);
    setResizable(false);
    
    // create application menu bar
    setJMenuBar(createMenuBar());
    
    // create new desktop pane object where internal jframes are going to be stored
    setContentPane(desktop);

    setIFrameConditions(iframe_login);

    // create login screen
    createLoginScreen();
  }
  
  private void setIFrameConditions(JInternalFrame iframe) {
      
    // disable border and frame top bar
    ((BasicInternalFrameUI)iframe.getUI()).setNorthPane(null);
    iframe.setBorder(null);
    
    desktop.add(iframe);
    
    // set internal screen to maximum size
    try {
      iframe.setMaximum(true);
    } catch(PropertyVetoException e){};
    
    
  }
  
  // create menu bar for application
  private JMenuBar createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu("File");
    JMenu helpMenu = new JMenu("Help");
    menuBar.add(fileMenu);
    menuBar.add(helpMenu);
    
    // menu bar items
    //JMenuItem _new = new JMenuItem("New");
    JMenuItem exit = new JMenuItem("Exit");
    JMenuItem help = new JMenuItem("Help");
    JMenuItem about = new JMenuItem("About");
    
    /*
    _new.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        //
      }
    });
    */
    
    exit.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        System.exit(0);
      }
    });
    
    help.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        JOptionPane.showMessageDialog(null, "Sorry, No Help Available", "Help on Using OpenSpace", JOptionPane.INFORMATION_MESSAGE);
      }
    });
    
    about.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        JOptionPane.showMessageDialog(null, "OpenSpace Beta", "About OpenSpace", JOptionPane.INFORMATION_MESSAGE);
      }
    });
    
   // fileMenu.add(_new);
    fileMenu.add(exit);
    helpMenu.add(help);
    helpMenu.add(about);
    
    return menuBar;
  }
  
  
  // create login area
  private void createLoginScreen() {
    iframe_login.show();
    JPanel content = new JPanel();
    content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
    content.setOpaque(false);
    
    // create login page contents
    JLabel name_label = new JLabel("Username:");
    content.add(name_label);
    
    final JTextField name_field = new JTextField(15);
    content.add(name_field);
    
    // add spacer
    content.add(Box.createRigidArea(new Dimension(0,5)));
    
    JLabel server_label = new JLabel("Server IP (Optional):");
    content.add(server_label);
    
    final JTextField server_field = new JTextField(15);
    content.add(server_field);
    
    // add spacer
    content.add(Box.createRigidArea(new Dimension(0,5)));
    
    JLabel port_label = new JLabel("Port:");
    content.add(port_label);
    
    final JTextField port_field = new JTextField(15);
    content.add(port_field);
    
    content.add(Box.createRigidArea(new Dimension(0,5)));
    
    JPanel buttonPane = new JPanel(new FlowLayout());

    // create submit button
    JButton submit = new JButton("Submit");
    buttonPane.add(submit);
    submit.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        user_name = name_field.getText();
        String new_ip = server_field.getText();
        String new_port = port_field.getText();
        
        // if user types in an optional server ip, connect to this server
        if (new_ip != null && new_ip.length() != 0 && new_port != null && new_port.length() != 0) {
          server_ip = new_ip;
          port = new_port;
        }
        
        if (user_name != null && user_name.length() != 0) {
          
          cn = new ClientNetwork(server_ip, port);
          
          // create rest of the screen
          createWaitScreen();
          
          
          // send user name to server - cn.sendMsg();
          iframe_login.hide();
          iframe_wait.show();
          setTitle("OpenSpace Beta [" + user_name +"]");
          cn.addObserver(ClientFrame.this);
        }
      }
    });
    
    // create cancel button
    JButton cancel = new JButton("Cancel");
    buttonPane.add(cancel);
    cancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        name_field.setText("");
        server_field.setText("");
        port_field.setText("");
      }
    });
    
    JPanel pane = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    pane.setLayout(gridbag);
    
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(content, c);
    pane.add(content);
    pane.add(buttonPane);

    iframe_login.setLayout(new GridBagLayout());
    iframe_login.add(pane);
  }
  
  
  // create wait room area
  private void createWaitScreen() {
    iframe_wait= new JInternalFrame("", false, false, false, false);
    setIFrameConditions(iframe_wait);
    createWaitroomButtons(iframe_wait);
    displayPlayRoomList(iframe_wait);
    
    JPanel topPanel = new JPanel();
    JPanel leftPanel = new JPanel();
    JPanel bottomPanel = new JPanel();
    
    // panels not defined yet
    iframe_wait.getContentPane().add(topPanel, BorderLayout.NORTH);
    iframe_wait.getContentPane().add(bottomPanel, BorderLayout.SOUTH);
    iframe_wait.getContentPane().add(leftPanel, BorderLayout.WEST);
  }
  
  // show on wait room screen the list of on-going games
  private void displayPlayRoomList(JInternalFrame iframe) {
    String[] columnNames = {"No.", "Title", "Users", "Status"};
    Object[][] cells = new Object[10][4];
    JTable table = new JTable(cells, columnNames);
    iframe.getContentPane().add(new JScrollPane(table), BorderLayout.CENTER);
  }
  
  // create play room area
  private void createPlayScreen() {
    iframe_play = new JInternalFrame("", false, false, false, false);
    setIFrameConditions(iframe_play);
    canvas = new DrawingCanvas(cn, user_name, game_mode);
    cn.addObserver(canvas);
    iframe_play.getContentPane().add(canvas, BorderLayout.CENTER);
    
    displayGamePlayers(iframe_play);
    
    JPanel naviPanel = new JPanel(new FlowLayout());
    JButton _return = new JButton("Return to Waitroom");
   
    _return.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        iframe_play.dispose();
        chatDisplay.setText("");
        cn.sendMsg(ClientModel.REMOVE_USER + "_" + user_name + "\r");
        iframe_wait.show();
      }
    });
    
    naviPanel.add(_return);
    
    JButton start = new JButton("Start Game");
    
    if (game_mode == ClientModel.ILLUSTRATOR) {
      start.addActionListener( new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          cn.sendMsg(ClientModel.START + "_\r");
          canvas.startDrawing();
        }
      }); 
      naviPanel.add(start);
    }
    
    JPanel bottomPanel = new JPanel();
    bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS) );
    
    if (game_mode == ClientModel.ILLUSTRATOR) {
      bottomPanel.add(displayIllustratorPanel(canvas));
    } else {
      bottomPanel.add(displayAssistantPanel(canvas));
    }
    
    // add chatbox
    createChatBox(bottomPanel);
    
    iframe_play.getContentPane().add(bottomPanel, BorderLayout.SOUTH);
    iframe_play.getContentPane().add(naviPanel, BorderLayout.NORTH);
  }
  
  // not yet implemented
  private void displayGamePlayers(JInternalFrame iframe) {
    JPanel rightPanel = new JPanel(new GridLayout(3,1));
    JPanel leftPanel = new JPanel(new GridLayout(3,1));
   
    for (int i=0; i<playerList.length; i++) {
      if (i < 3) {
        addPlayers("", leftPanel, i);
      } else {
        addPlayers("", rightPanel, i);
      }
    }
   
    iframe.getContentPane().add(rightPanel, BorderLayout.EAST);
    iframe.getContentPane().add(leftPanel, BorderLayout.WEST);
  }
  
  private void addPlayers(String str, JPanel panel, int i) {
    JTextField player = new JTextField(7);
    player.setEditable(false);
    player.setText(str);
    player.setHorizontalAlignment(JTextField.CENTER ); 
    player.setBorder( BorderFactory.createCompoundBorder(
                          BorderFactory.createCompoundBorder(
                          BorderFactory.createTitledBorder(""),
                          BorderFactory.createEmptyBorder(5,5,5,5)),
                          player.getBorder()));
    playerList[i] = player;
    panel.add(player);
  }
  
  private void updatePlayerList(String str, int i) {
    playerList[i].setText(str);
  }
  
  private int findBlankSpot() {
    for (int i=0; i<6; i++) {
      if (playerList[i].getText().length() == 0) {
        return i;
      }
    }
    return -1;
  }
  
  
  
  private void createChatBox(JPanel bottomPanel) {
    JPanel chatBox = new JPanel(new FlowLayout());
     // create login page contents
    JLabel chat_label = new JLabel("ChatBox:");
    //name_label.setForeground(Color.white);
    chatBox.add(chat_label);
    
    final JTextField chat_field = new JTextField(30);
    chatBox.add(chat_field);
    
    JButton send = new JButton("Send");
    
    send.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        String msg = chat_field.getText();
        userTalked(user_name, msg);
        cn.sendMsg(ClientModel.CHAT + "_" + user_name + "_" + msg + "_\r");
        chat_field.setText("");
      }
    }); 
    
    chatBox.add(send);

    bottomPanel.add(chatBox);
    
    chatDisplay.setLineWrap(true);
    chatDisplay.setWrapStyleWord(true);
    chatDisplay.setEditable(false);
    
    JScrollPane scrollPane = new JScrollPane(chatDisplay);
    scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    scrollPane.setPreferredSize(new Dimension(473, 100));
    scrollPane.setMaximumSize(new Dimension(473, 100));
    scrollPane.setMinimumSize(new Dimension(473, 100));
    
    scrollPane.setBorder( BorderFactory.createCompoundBorder(
                          BorderFactory.createCompoundBorder(
                          BorderFactory.createTitledBorder("Client Chat Dialogue"),
                          BorderFactory.createEmptyBorder(5,5,5,5)),
                          scrollPane.getBorder()));
        
    bottomPanel.add(scrollPane);
  }
  
  private void userTalked(String user, String msg) {
    chatDisplay.append(user + ": " + msg +"\n");
    
  }
  
  private JPanel displayAssistantPanel(final DrawingCanvas canvas) {
    JPanel buttonPanel = new JPanel();    
    buttonPanel.setLayout(new FlowLayout());

    // buttons to change color
    ButtonGroup group1 = new ButtonGroup();
    makeColorButton("Black", Color.BLACK, group1, buttonPanel, canvas);
    makeColorButton("Red", Color.RED, group1, buttonPanel, canvas);
    makeColorButton("Blue", Color.BLUE, group1, buttonPanel, canvas);
    makeColorButton("Green", Color.GREEN, group1, buttonPanel, canvas);
    makeColorButton("Yellow", Color.YELLOW, group1, buttonPanel, canvas);
    
    return buttonPanel;
  }
  
  private JPanel displayIllustratorPanel(final DrawingCanvas canvas) {
    JPanel buttonPanel = new JPanel();    
    buttonPanel.setLayout(new FlowLayout());
    ButtonGroup group2 = new ButtonGroup();
    makeColorButton("Erase", Color.WHITE, group2, buttonPanel, canvas);
    
    /*
    // buttons to change color
    ButtonGroup group1 = new ButtonGroup();
    makeColorButton("Black", Color.BLACK, group1, buttonPanel, canvas);
    makeColorButton("Red", Color.RED, group1, buttonPanel, canvas);
    makeColorButton("Blue", Color.BLUE, group1, buttonPanel, canvas);
    makeColorButton("Green", Color.GREEN, group1, buttonPanel, canvas);
    makeColorButton("Yellow", Color.YELLOW, group1, buttonPanel, canvas);
    makeColorButton("Erase", Color.WHITE, group1, buttonPanel, canvas);
    */
    
    // slider to change paint brush size
    final JSlider sizeSlider = new JSlider(1, 20, 10);
    canvas.setSize(10);
    sizeSlider.addChangeListener(new ChangeListener() {
      public void stateChanged (ChangeEvent event) {
        canvas.setSize(sizeSlider.getValue());
      }
    });
    
    buttonPanel.add(sizeSlider);    
    
    JButton erase_all = new JButton("Erase All");
    erase_all.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        canvas.clearScreen();
      }
    });
    
    buttonPanel.add(erase_all);
    
    return buttonPanel;
  }
  
  
  // create buttons for wait room
  private void createWaitroomButtons(JInternalFrame iframe) {
    JPanel rightPanel = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    rightPanel.setLayout(gridbag);
    
    JButton create = new JButton("Create");
    JButton join = new JButton("Join");
    JButton options = new JButton("Options");
    JButton quit = new JButton("Quit");
    
    create.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        room_title = JOptionPane.showInputDialog("Please Enter Room Title");
        if (room_title != null && room_title.length()  != 0) {
          game_mode = ClientModel.ILLUSTRATOR;
          cn.sendMsg(ClientModel.CREATE + "_\r");
          createPlayScreen();
          updatePlayerList(user_name, 0);
          iframe_wait.hide();
          iframe_play.show();
        }
      }
    });
    
    join.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        game_mode = ClientModel.ASSISTANT;
        cn.sendMsg(ClientModel.JOIN + "_" + user_name + "_\r");
        createPlayScreen();
        iframe_wait.hide();
        iframe_play.show();
      }
    });
    
    options.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        JOptionPane.showMessageDialog(null, "Not Available", "Options", JOptionPane.INFORMATION_MESSAGE);
      }
    });                       
    
    quit.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        System.exit(0);
      }
    });
    
    c.fill = GridBagConstraints.BOTH;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.insets = new Insets(2,5,2,5);
    c.anchor = GridBagConstraints.NORTH;

    gridbag.setConstraints(create, c);
    rightPanel.add(create);
    gridbag.setConstraints(join, c);
    rightPanel.add(join);
    gridbag.setConstraints(options, c);
    rightPanel.add(options);
    gridbag.setConstraints(quit, c);
    rightPanel.add(quit);
    
    JPanel outerPanel = new JPanel(new BorderLayout());
    outerPanel.add(rightPanel, BorderLayout.NORTH);
    
    iframe.getContentPane().add(outerPanel, BorderLayout.EAST);
  }
  
  // make color changing buttons
  private void makeColorButton(String label, final Color c, ButtonGroup group, JPanel buttonPanel, final DrawingCanvas canvas) {
    // select first button in group
    boolean selected = group.getButtonCount() == 0;
    JRadioButton button = new JRadioButton(label, selected);
    buttonPanel.add(button);
    group.add(button);
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        canvas.setColor(c);
        cn.sendMsg(ClientModel.COLOR_CHANGE + "_" + c.getRGB() + "_\r");
      }
    });
  }
  
  // not implemented yet
  public void update(Observable o, Object arg) {
    String msg = cn.getMsg();
    String[] loc = msg.split("_");
    int key = Integer.parseInt(loc[0]);
    
    if (game_mode == ClientModel.ILLUSTRATOR || game_mode == ClientModel.ASSISTANT) {
    
    if (key == ClientModel.CHAT) {
      userTalked(loc[1], loc[2]);
    } else if(key == ClientModel.JOIN) {
      if (game_mode == ClientModel.ILLUSTRATOR) {
        updatePlayerList(loc[1], findBlankSpot());
        String playersInfo = ClientModel.ADD_USER + "_";
        
        for (int i=0; i<6;  i++) {
          if (playerList[i].getText().length() != 0) {
            playersInfo = playersInfo + playerList[i].getText() + "_" + i + "_";
          }
        }
        cn.sendMsg(playersInfo + "\r");     
      }
    }else if( key ==ClientModel.ADD_USER) {
        for (int i=1; i<loc.length; i+=2) {
          updatePlayerList(loc[i], Integer.parseInt(loc[i+1]));
        }
        
    } else if (key == ClientModel.REMOVE_USER) {
      for (int i=0; i<6; i++) {
        if ((playerList[i].getText()).equals(loc[1])) {
          playerList[i].setText("");
        }
      }
    }
    }
  }
}