import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
import java.awt.image.BufferedImage;

// this is the drawing canvas
public class DrawingCanvas extends JPanel implements Observer {
  
  private Color penColor = Color.BLACK;
  private int penSize;
  private ClientNetwork cn;
  private String user_name;
  
  private int prevX = 0;
  private int prevY = 0;
  private int currX;
  private int currY;

  private BufferedImage _bufImage = null;
  
  private int game_mode; // 1 for illustrator, 0 for assistant
  
  public DrawingCanvas(final ClientNetwork cn, String user_name, int game_mode) {
    setBackground(Color.WHITE);
    
    this.cn = cn;
    this.user_name = user_name;
    this.game_mode = game_mode;

    setBackground(Color.WHITE);
    this.repaint();
  }
  
  public void startDrawing() {
    addMouseListener(new MouseAdapter() {
      public void mouseReleased(MouseEvent event) {
        myMouseReleasedHandler();
      }
    });
    
    addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseDragged(MouseEvent event) {
        myMouseDragPaint(event);
      }
    });
  }
  
  private void myMouseDragPaint(java.awt.event.MouseEvent event) {
    currX = event.getX();
    currY = event.getY();
    
    //Previous coordinates will only be 0,0 if this is a new line.
    //If this is the case, set previous coords equal to new coords
    //which will begin a new line.
    if (prevX == 0) { 
      prevX = currX;
    }
    if (prevY == 0) {
      prevY = currY; 
    }

   
    
    Graphics2D grafarea = _bufImage.createGraphics();
    drawCurrentShape(grafarea);

    this.repaint();
    cn.sendMsg(ClientModel.DRAW + "_" + prevX + "_" + prevY + "_" + currX + "_" + currY + "_" + penSize + "_" + penColor.getRGB() + "\r");
    
    prevX = currX;
    prevY = currY;
  }
  
  private void myMouseReleasedHandler() {
    //Reset coordinates to 0,0 indicating that the line has been broken
    prevX = 0;
    prevY = 0;
  }
  
  // draws shape
  private void drawCurrentShape(Graphics2D g2) {
    BasicStroke stroke = new BasicStroke((float)penSize, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    g2.setStroke(stroke);
    g2.setColor(penColor);
    g2.drawLine(prevX, prevY, currX, currY);
  }
  
  // paint component
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    Graphics2D g2 = (Graphics2D)g;  // downcast to Graphics2D

    if (_bufImage == null) {
      //this is the first time, initialize _bufImage
      int w = this.getWidth();
      int h = this.getHeight();
      _bufImage = (BufferedImage)this.createImage(w, h);
      Graphics2D gc = _bufImage.createGraphics();
      gc.setColor(Color.white);
      gc.fillRect(0, 0, w, h); // fill in background
    }
    g2.drawImage(_bufImage, null, 0, 0);  // draw previous shapes
    
    if (prevX != 0 && prevY != 0 ) {
      drawCurrentShape(g2);
    }
  }

  
  // used to draw on canvas by listening to server for drawing coordinates only if you are an assistant
  public void update(Observable o, Object arg) {
      String msg = cn.getMsg();
      String[] loc = msg.split("_");
      int key = Integer.parseInt(loc[0]);
      
      if (game_mode == ClientModel.ASSISTANT && (key == ClientModel.DRAW || key == ClientModel.ERASE)) {
        
        switch(key) {
          case ClientModel.DRAW: {
            prevX = Integer.parseInt(loc[1]);
            prevY = Integer.parseInt(loc[2]);
            currX = Integer.parseInt(loc[3]);
            currY = Integer.parseInt(loc[4]);
            penSize = Integer.parseInt(loc[5]);
            penColor = new Color(Integer.parseInt(loc[6]));
            Graphics2D grafarea = _bufImage.createGraphics();
            drawCurrentShape(grafarea);
            this.repaint();
            break;
          }
          case ClientModel.ERASE: {
            clearScreen();
            break;
          }
        }
      } else if (game_mode == ClientModel.ILLUSTRATOR) {
        if (key == ClientModel.COLOR_CHANGE) {
          penColor = new Color(Integer.parseInt(loc[1]));
        }
      }
  }
               
  
  // change color of the pen
  public void setColor(Color c) {
    penColor = c;
  }
  
  // clear screen
  public void clearScreen() {
    myMouseReleasedHandler();
    _bufImage = null;
    this.repaint();
    
    if (game_mode == ClientModel.ILLUSTRATOR ) {
      cn.sendMsg(ClientModel.ERASE + "\r");
    }
    
  }
  
  public void setSize(int s) {
    penSize = s;
  }
  
  
}
