import java.util.*;

/**
 * Main Room for Users
 * This is the main control for the server.
 * It holds a collection of clients in the main room and
 * available games to connect to.
 * @author David Couvrette
 * @version 0.1 pre-beta release
 */
public class MainRoom implements Room {
  
  // hashmap to store all the clients that are in the main server room
  private HashMap clients;
  // hashmap to store all the current games on this server
  private HashMap games;
  
  
  /**
   * Construct and initialize the server
   */
  public MainRoom() {
    clients = new HashMap();
    games = new HashMap();
    ScanIncomingConnections scan = new ScanIncomingConnections(this);
    Thread t = new Thread(scan);
    t.start();
  }
  
  /**
   * Adds a new client to the set of current clients
   * Starts a new thread that waits for another client
   * @param c The new client that is joining the server
   */  
  public void addNewClient(Client newClient) {
    if (clients.containsKey(newClient.getUserName()))
      throw new IllegalArgumentException("That name is already taken");
    clients.put(newClient.getUserName(), newClient);
    newClient.setRoom(this);
  }
  
  public boolean sendTo(String userName, String msg) {
    Client c = (Client)clients.get(userName);
    return c.sendMsg(msg);
  }
  
  /**
   * Sends a message to all clients
   * @param message The message that is sent to every client
   */
  public void sendToAllClients(String message) {
    Collection col = clients.values();
    Iterator it = col.iterator();
    while(it.hasNext()) {
      Client c = (Client)it.next();
      c.sendMsg(message);
    }
  }
  
  /**
   * Remove a client from the room.
   * This room does not have any direct reference to
   * a client after it has been removed
   * @param userName The name of the client that is being removed
   * @return The client that has just been removed
   */
  public Client removeClient(String userName) {
    return (Client)clients.remove(userName);
  }
  
  /**
   * handles messages / commands from client
   * @param msg the message or command that is being sent
   * @param sender the client who sent the message
   * @return true if the command succeeded, otherwise false
   * maybe later better define what it means to succeed.
   */
  public boolean handleMsg(String msg, Client sender) {
    // temporary... forwards message to all other clients
    Set allClients = clients.keySet();
    Iterator it = allClients.iterator();
    boolean fail = false;
    while (it.hasNext()) {
      String user = (String) it.next();
      if (!user.equals(sender.getUserName()))
        sendTo(user, msg);
    }
    return !fail;
  }
  
  /**
   * Checks to see if a given user is in the room.
   * @param userName the user name to search for
   * @param true if the room contains the given user name, otherwise false
   */
  public boolean containsUser(String userName) {
    return clients.containsKey(userName);
  }
  
    //MIKIKO
  // starts the server
   public static void main(String[] args)
  {
      MainRoom test = new MainRoom();
  }
  //END_MIKIKO
}
