/**
 * Every type of room where a client can be needs
 * to be able to handle incoming messages.
 * @author David Couvrette
 * @version 0.1 pre-beta release
 * 
 * Both the game room and the main server room will implement
 * this interface.  Both of these rooms have similarities such
 * as handling messages and adding and removing clients.
 * 
 */
public interface Room {
  
  /**
   * Handles a message from the client.
   * @param msg message from client
   * @param sender the client that send the message
   * @return true if message was accepted, otherwise false
   */
  public boolean handleMsg(String msg, Client sender);
  
  /**
   * Adds a new client to the room
   * @param newClient The client that is to be added to the room
   */
  public void addNewClient(Client newClient);
  
  /**
   * Remove a client from the room.
   * This room does not have any direct reference to
   * a client after it has been removed
   * @param userName The name of the client that is being removed
   * @return The client that has just been removed, returns null if
   * the given user name is not in the room
   */
  public Client removeClient(String userName);
  
  /**
   * Checks to see if a given user is in the room.
   * @param userName the user name to search for
   * @param true if the room contains the given user name, otherwise false
   */
  public boolean containsUser(String userName);
}
